/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.Loader;

/**
 * Implementation of Factory pattern responsible for instantiating
 * the FTHPISDataStore interface implementation.
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class WSContextDataStoreFactory
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(WSContextDataStoreFactory.class);

  // Authenticator property key & default implementation
  private static final String IMPL_KEY = "juddi.wscontextDataStore";

  private static final String DEFAULT_IMPL = "cgl.fthpis.datastore.jdbc.WSContextJDBCDataStore";

  private static Class implClass = null;

  /**
   * Returns a new instance of a FTHPISDataStore.
   *
   * @return FTHPISDataStore
   */
  public static FTHPISDataStore getDataStore()
  {
    FTHPISDataStore dataStore = null;

    try
    {
      // make sure we know what class to create
      if (implClass == null)
        implClass = loadImplClass();

      // true if a configuration problem exists
      if (implClass == null)
        throw new RegistryException("The registry is not configured " +
            "correctly.");

      // try to instantiate a new FTHPISDataStore
      dataStore = (FTHPISDataStore)implClass.newInstance();
    }
    catch(Exception e)
    {
      log.error("Exception while attempting to instantiate the " +
        "implementation of FTHPISDataStore: " + implClass.getName() +
        "\n" + e.getMessage());
      log.error(e);
    }

    return dataStore;
  }

  /**
   *
   */
  private static synchronized Class loadImplClass()
  {
    if (implClass != null)
      return implClass;

    // grab class name of the FTHPISDataStore implementation to create
    String className = Config.getStringProperty(IMPL_KEY,DEFAULT_IMPL);

    // write the FTHPISDataStore implementation name to the log
    log.info("FTHPISDataStore Implementation = " + className);

    try
    {
      // Use Loader to locate & load the FTHPISDataStore implementation
      implClass = Loader.getClassForName(className);
    }
    catch(ClassNotFoundException e)
    {
      log.error("The registry is not configured correctly. The specified " +
                "FTHPISDataStore class '" + className + "' was not found in " +
                "classpath.");
      log.error(e);
    }

    return implClass;
  }

  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
  {
    // delecare work variables
    int connCount = 1;
    FTHPISDataStore[] stores = new FTHPISDataStore[connCount];

    // aquire some connections
    for (int i=0; i<connCount; i++)
    {
      stores[i] = (FTHPISDataStore)WSContextDataStoreFactory.getDataStore();
      if (stores[i] != null)
        log.info("Got a FTHPISDataStore: "+stores[i].getClass().getName());
      else
        log.info("Sorry - A FTHPISDataStore object could not be created.");
    }

    // release those connections
    for (int i=0; i<connCount; i++)
    {
      if (stores[i] != null)
      {
        stores[i].release();
        log.info("FTHPISDataStore "+i+" released.");
      }
      else
        log.info("FTHPISDataStore "+i+" was never successfully created.");
    }
  }
}
