/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.Iterator;

import cgl.util.jdbc.ConnectionManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.DynamicQuery;
import org.apache.juddi.util.jdbc.Transaction;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class FindContextBySessionAndServiceKeyQuery {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(
            FindContextBySessionAndServiceKeyQuery.class);

    static String selectSQL;
    static {
        // build selectSQL
        StringBuffer sql = new StringBuffer(200);
        sql.append("SELECT T.CONTEXT_KEY ");
        sql.append(
                "FROM SESSION_CONTEXT S, SERVICE_CONTEXT P, CONTEXT_ENTITY T ");

        selectSQL = sql.toString();
    }

    /**
     * Select ...
     *
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector select(String sessionKey,
                                String serviceKey,
                                Vector keysIn,
                                FindQualifiers qualifiers,
                                Connection connection) throws java.sql.
            SQLException {
        log.info(
                "cgl.fthpis.datastore.jdbc.FindContextBySessionKeyQuery => select(...)");
        // if there is a keysIn vector but it doesn't contain
        // any keys then the previous query has exhausted
        // all possibilities of a match so skip this call.
        if ((keysIn != null) && (keysIn.size() == 0)) {
            return keysIn;
        }

        Vector keysOut = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        // construct the SQL statement
        DynamicQuery sql = new DynamicQuery(selectSQL);
        appendWhere(sql, sessionKey, serviceKey, qualifiers);
        appendIn(sql, keysIn);
        //appendOrderBy(sql,qualifiers);

        try {
            log.info("sql statement = " + sql.toString());
            statement = sql.buildPreparedStatement(connection);
            resultSet = statement.executeQuery();

            while (resultSet.next()) {
                keysOut.addElement(resultSet.getString(1)); //("PROPERTY_KEY"));
            }

            return keysOut;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the ResultSet: " + e.getMessage());
            }

            try {
                statement.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Statement: " + e.getMessage());
            }
        }
    }

    /**
     *
     */
    private static void appendWhere(DynamicQuery sql,
                                    String sessionKey,
                                    String serviceKey,
                                    FindQualifiers qualifiers) {
        sql.append(
                "WHERE (T.CONTEXT_KEY=S.CONTEXT_KEY) AND (T.CONTEXT_KEY=P.CONTEXT_KEY) ");

        if (sessionKey != null) {
            sql.append("AND S.SESSION_KEY = ? ");
            sql.addValue(sessionKey);
        }
        if (serviceKey != null) {
            sql.append("AND P.SERVICE_KEY = ? ");
            sql.addValue(serviceKey);
        }

    }

    /**
     * Utility method used to construct SQL "IN" statements such as
     * the following SQL example:
     *
     *   SELECT * FROM TABLE WHERE MONTH IN ('jan','feb','mar')
     *
     * @param sql StringBuffer to append the final results to
     * @param keysIn Vector of Strings used to construct the "IN" clause
     */
    private static void appendIn(DynamicQuery sql, Vector keysIn) {
        if (keysIn == null) {
            return;
        }

        sql.append("AND T.CONTEXT_KEY IN (");

        int keyCount = keysIn.size();
        for (int i = 0; i < keyCount; i++) {
            String key = (String) keysIn.elementAt(i);
            sql.append("?");
            sql.addValue(key);

            if ((i + 1) < keyCount) {
                sql.append(",");
            }
        }

        sql.append(") ");
    }

    /**
     *
     */
    private static void appendOrderBy(DynamicQuery sql,
                                      FindQualifiers qualifiers) {
        sql.append("ORDER BY ");

        if (qualifiers == null) {
            sql.append("T.LAST_UPDATE DESC");
        } else if (qualifiers.sortByDateAsc) {
            sql.append("T.LAST_UPDATE ASC");
        } else {
            sql.append("T.LAST_UPDATE DESC");
        }
    }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection("wscontext");
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {
        String sessionKey = "913C9280-49C2-11DA-A453-A85F9613C9BF";
        String serviceKey = "9459ED10-4B4C-11DA-AD10-A6E6C716FDE9";

//    Vector keysIn = new Vector();
//    keysIn.add("61004340-3A9B-11DA-8340-D76632EAED77");

        Transaction txn = new Transaction();

        if (connection != null) {
            try {
                // begin a new transaction
                txn.begin(connection);

//        Vector vector = select(sessionKey,keysIn,null,connection);
                Vector vector = select(sessionKey, serviceKey, null, null,
                                       connection);
                log.info("debug point in main - 2");

                for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
                    Object item = (Object) iter.next();
                    log.info("contextKey = " + item.toString());

                }
                log.info("debug point in main - 3");

                // commit the transaction
                txn.commit();
            } catch (Exception ex) {
                try {
                    txn.rollback();
                } catch (java.sql.SQLException sqlex) {
                    sqlex.printStackTrace();
                }
                throw ex;
            }
        }
    }
}
