/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;

import cgl.util.jdbc.ConnectionManager;
import cgl.fthpis.datatype.publisher.Publisher;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.DynamicQuery;
import org.apache.juddi.util.jdbc.Transaction;


/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class FindPublisherByNameQuery {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(FindPublisherByNameQuery.class);

    static String selectSQL;
    static {
        // build selectSQL
        StringBuffer sql = new StringBuffer(200);
        sql.append("SELECT P.PUBLISHER_ID,P.PUBLISHER_NAME ");
        sql.append("FROM PUBLISHER P ");
        selectSQL = sql.toString();
    }

    /**
     * Select ...
     *
     * @param name primary key value
     * @param keysIn primary key value
     * @param qualifiers primary key value
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector select(String name,
                                Vector idsIn,
                                FindQualifiers qualifiers,
                                Connection connection) throws java.sql.
            SQLException {
        // if there is a idsIn vector but it doesn't contain
        // any publisher IDs then the previous query has exhausted
        // all possibilities of a match so skip this call.
        if ((idsIn != null) && (idsIn.size() == 0)) {
            return idsIn;
        }

        Vector idsOut = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        // construct the SQL statement
        DynamicQuery sql = new DynamicQuery(selectSQL);
        appendWhere(sql, name, qualifiers);
        appendIn(sql, name, idsIn);
        appendOrderBy(sql, qualifiers);

        try {
            log.debug(sql.toString());

            statement = sql.buildPreparedStatement(connection);
            resultSet = statement.executeQuery();

            while (resultSet.next()) {
                idsOut.addElement(resultSet.getString(1)); //("PUBLISHER_ID"));
            }

            return idsOut;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Find Publisher ResultSet: " + e.getMessage(), e);
            }

            try {
                statement.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Find Publisher Statement: " + e.getMessage(), e);
            }
        }
    }

    /**
     *
     */
    private static void appendWhere(DynamicQuery sql, String name,
                                    FindQualifiers qualifiers) {
        if ((name == null) || (name.length() == 0)) {
            return;
        }

        if ((qualifiers != null) && (qualifiers.exactNameMatch)) {

            sql.append("WHERE P.PUBLISHER_NAME = ? ");
            sql.addValue(name);
        } else {
            sql.append("WHERE P.PUBLISHER_NAME LIKE ? ");
            sql.addValue(name + "%");
        }
    }

    /**
     * Utility method used to construct SQL "IN" statements such as
     * the following SQL example:
     *
     *   SELECT * FROM TABLE WHERE MONTH IN ('jan','feb','mar')
     *
     * @param sql StringBuffer to append the final results to
     * @param keysIn Vector of Strings used to construct the "IN" clause
     */
    private static void appendIn(DynamicQuery sql, String name, Vector keysIn) {
        if (keysIn == null) {
            return;
        }

        if ((name == null) || (name.length() == 0)) {
            sql.append("WHERE P.PUBLISHER_ID IN (");
        } else {
            sql.append("AND P.PUBLISHER_ID IN (");
        }

        int keyCount = keysIn.size();
        for (int i = 0; i < keyCount; i++) {
            String key = (String) keysIn.elementAt(i);
            sql.append("?");
            sql.addValue(key);

            if ((i + 1) < keyCount) {
                sql.append(",");
            }
        }

        sql.append(") ");
    }

    /**
     * if the qualifier parameter is null or if the
     * sortByNameDesc qualifier is true or if neither of
     * the sortByNameXxxx qualifiers are true then sort
     * the result by name in decending order.
     */
    private static void appendOrderBy(DynamicQuery sql,
                                      FindQualifiers qualifiers) {
        sql.append("ORDER BY ");

        if ((qualifiers == null) ||
            (qualifiers.sortByNameDesc) ||
            ((!qualifiers.sortByNameAsc) && (!qualifiers.sortByNameDesc))) {
            sql.append("P.PUBLISHER_NAME DESC");
        } else if (qualifiers.sortByNameAsc) {
            sql.append("P.PUBLISHER_NAME ASC");
        }
    }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection("wscontext");
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {
        String name = new String("%");

        Vector idsIn = null;
        idsIn = new Vector();
        idsIn.add(new String("uddi_user"));

        Transaction txn = new Transaction();

        if (connection != null) {
            try {
                // begin a new transaction
                txn.begin(connection);

                Vector results = select(name, null, null, connection);
                if (results != null) {
                    for (int i = 0; i < results.size(); i++) {
                        log.info(i + ": " + (String) results.elementAt(i));
                        Publisher publisher = PublisherTable.select((String)
                                results.
                                elementAt(i), connection);
                    }
                }

                // commit the transaction
                txn.commit();
            } catch (Exception ex) {
                try {
                    txn.rollback();
                } catch (java.sql.SQLException sqlex) {
                    sqlex.printStackTrace();
                }
                throw ex;
            }
        }
    }
}
