/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.fthpis.datatype.attribute.Attribute;
import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.AttributeName;
import cgl.fthpis.datatype.attribute.AttributeValueType;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class ServiceAttributeTable {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(ServiceAttributeTable.class);

    static String insertSQL = null;
    static String deleteSQL = null;
    static String selectSQL = null;
    static String selectByServiceKeySQL = null;
    static String deleteByServiceKeySQL = null;
    static String verifyOwnershipSQL = null;

    static {
        // buffer used to build SQL statements
        StringBuffer sql = null;

        // build insertSQL
        sql = new StringBuffer(100);
        sql.append("INSERT INTO SERVICE_PROPERTY (");
        sql.append("SERVICE_KEY,");
        sql.append("PROPERTY_KEY,");
        sql.append("LAST_UPDATE) ");
        sql.append("VALUES (?,?,?)");
        insertSQL = sql.toString();

        // build deleteSQL
        sql = new StringBuffer(100);
        sql.append("DELETE FROM SERVICE_PROPERTY ");
        sql.append("WHERE PROPERTY_KEY=?");
        deleteSQL = sql.toString();

        // build selectSQL
        sql = new StringBuffer(100);
        sql.append("SELECT ");
        sql.append("SERVICE_KEY ");
        sql.append("FROM SERVICE_PROPERTY ");
        sql.append("WHERE PROPERTY_KEY=?");
        selectSQL = sql.toString();

        // build selectByBusinessKeySQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("PROPERTY_KEY ");
        sql.append("FROM SERVICE_PROPERTY ");
        sql.append("WHERE SERVICE_KEY=?");
        selectByServiceKeySQL = sql.toString();

        // build deleteByBusinessKeySQL
        sql = new StringBuffer(100);
        sql.append("DELETE FROM SERVICE_PROPERTY ");
        sql.append("WHERE SERVICE_KEY=?");
        deleteByServiceKeySQL = sql.toString();

        // build verifyOwnershipSQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("* ");
        sql.append("FROM BUSINESS_SERVICE e, SERVICE_PROPERTY s ");
        sql.append("WHERE e.SERVICE_KEY = s.SERVICE_KEY ");
        sql.append("AND s.PROPERTY_KEY=? ");
        sql.append("AND e.PUBLISHER_ID=?");
        verifyOwnershipSQL = sql.toString();
    }

    /**
     * Insert new row into the SERVICE_PROPERTY table.
     *
     * @param attribute Attribute holding values to be inserted
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static void insert(Attribute attribute,
                              Connection connection) throws
            java.sql.SQLException {
        log.info(
                "cgl.fthpis.datastore.jdbc.SessionService.insert() function called.");
        PreparedStatement statement = null;
        Timestamp timeStamp = new Timestamp(System.currentTimeMillis());
        try {
            statement = connection.prepareStatement(insertSQL);
            statement.setString(1, attribute.getServiceKey().toString());
            statement.setString(2, attribute.getAttributeKey().toString());
            statement.setTimestamp(3, timeStamp);

            log.debug("insert into SERVICE_PROPERTY table:\n\n\t" + insertSQL +
                      "\n\t SERVICE_KEY=" + attribute.getServiceKey().toString() +
                      "\n\t PROPERTY_KEY=" +
                      attribute.getAttributeKey().toString() +
                      "\n\t LAST_UPDATE=" + timeStamp.getTime() + "\n");

            statement.executeUpdate();
        } finally {
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     * Delete row from the SERVICE_PROPERTY table.
     *
     * @param serviceKey Primary key value
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static void delete(String attributeKey, Connection connection) throws
            java.sql.SQLException {
        PreparedStatement statement = null;

        try {
            statement = connection.prepareStatement(deleteSQL);
            statement.setString(1, attributeKey.toString());

            log.debug("delete from SERVICE_PROPERTY table:\n\n\t" + deleteSQL +
                      "\n\t PROPERTY_KEY=" + attributeKey.toString() + "\n");

            // execute
            statement.executeUpdate();
        } finally {
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     * Select one row from the SERVICE_PROPERTY table.
     *
     * @param serviceKey primary key value
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector select(String attributeKey, Connection connection) throws
            java.sql.SQLException {
        Vector service_vector = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            statement = connection.prepareStatement(selectSQL);
            statement.setString(1, attributeKey.toString());

            log.debug("select from SERVICE_PROPERTY table:\n\n\t" + selectSQL +
                      "\n\t PROPERTY_KEY=" + attributeKey.toString() + "\n");

            resultSet = statement.executeQuery();
            if (resultSet.next()) {
                service_vector.add(resultSet.getString(1));
            }

            return service_vector;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) { /* ignored */}
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     * Delete multiple rows from the SERVICE_PROPERTY table that are assigned to
     * the serviceKeu specified.
     *
     * @param serviceKey String
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static void deleteByServiceKey(String serviceKey,
                                          Connection connection) throws java.
            sql.SQLException {
        PreparedStatement statement = null;

        try {
            // prepare the delete
            statement = connection.prepareStatement(deleteByServiceKeySQL);
            statement.setString(1, serviceKey.toString());

            log.debug("delet from the SERVICE_PROPERTY table:\n\n\t" +
                      deleteByServiceKeySQL +
                      "\n\t SERVICE_KEY=" + serviceKey.toString() + "\n");

            // execute
            statement.executeUpdate();
        } finally {
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     * Select all rows from the service_property table for
     * a given serviceKey.
     *
     * @param serviceKey String
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector selectByServiceKey(String serviceKey,
                                            Connection connection) throws java.
            sql.SQLException {
        Vector serviceList = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            // create a statement to query with
            statement = connection.prepareStatement(selectByServiceKeySQL);
            statement.setString(1, serviceKey.toString());

            log.debug("select from SERVICE_PROPERTY table:\n\n\t" +
                      selectByServiceKeySQL +
                      "\n\t SERVICE_KEY=" + serviceKey.toString() + "\n");

            // execute the statement
            resultSet = statement.executeQuery();
            while (resultSet.next()) {
                Attribute context = new Attribute();
                context.setServiceKey(serviceKey);
                context.setAttributeKey(resultSet.getString(1));
                serviceList.add(context);
            }

            return serviceList;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) { /* ignored */}
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     * Verify that 'authorizedName' has the authority to update or delete
     * BusinessService identified by the serviceKey parameter
     *
     * @param serviceKey
     * @param publisherID
     * @param connection
     * @throws java.sql.SQLException
     */
    public static boolean verifyOwnership(String serviceKey, String publisherID,
                                          Connection connection) throws java.
            sql.SQLException {
        if ((serviceKey == null) || (publisherID == null)) {
            return false;
        }

        boolean authorized = false;
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            statement = connection.prepareStatement(verifyOwnershipSQL);
            statement.setString(1, serviceKey);
            statement.setString(2, publisherID);

            log.debug("checking ownership of SERVICE_PROPERTY:\n\n\t" +
                      verifyOwnershipSQL +
                      "\n\t PROPERTY_KEY=" + serviceKey +
                      "\n\t PUBLISHER_ID=" + publisherID + "\n");

            resultSet = statement.executeQuery();
            if (resultSet.next()) {
                authorized = true;
            }

            return authorized;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) { /* ignored */}
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection("uddi");
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {
        Transaction txn = new Transaction();
        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

        if (connection != null) {
            try {
                String serviceKey = uuidgen.uuidgen_with_timestamp();

                String attributeKey = uuidgen.uuidgen();

                // begin a new transaction
                txn.begin(connection);

                Attribute attribute = new Attribute();
                attribute.setAttributeKey(attributeKey);
                attribute.setServiceKey("77E88540-3A9B-11DA-8540-B3035B263F99");
                attribute.setAttributeName(new AttributeName("cpu_deneme"));
                attribute.setAttributeValueType(new AttributeValueType("number"));
                attribute.setAttributeValue(new AttributeValue("3"));
                Lease lease = new Lease();
                lease.setIsInfinite("false");
                lease.setLeaseTimeOut("10000000000000000");
                attribute.setLease(lease);

                ServiceAttributeTable.insert(attribute, connection);

                Vector service_key_vector = ServiceAttributeTable.select(attributeKey,
                        connection);
                for (Iterator iter = service_key_vector.iterator(); iter.hasNext(); ) {
                    String item = (String) iter.next();
                    if (item != null) {
                        log.info("service key = " + item);
                    }
                }

                ServiceAttributeTable.delete(attributeKey, connection);
//
//        // insert another new SessionService
//        service.setServiceKey(uuidgen.uuidgen());
//        SessionServiceTable.insert(service,connection);
//
//        // insert one more new SessionService
//        service.setServiceKey(uuidgen.uuidgen());
//        SessionServiceTable.insert(service,connection);
//
//        // select a SessionService object
//        service = SessionServiceTable.select(serviceKey,connection);

//        // delete a SessionService object
//        SessionServiceTable.delete(serviceKey,connection);
//
//        // select a SessionService object
//        service = SessionServiceTable.select(serviceKey,connection);
//
//        // select a Collection SessionService objects by SessionKey
//        SessionServiceTable.selectByBusinessKey(businessKey,connection);
//
//        // delete a Collection SessionService objects by SessionKey
//        SessionServiceTable.deleteByBusinessKey(businessKey,connection);
//
//        // select a Collection SessionService objects by SessionKey
//        SessionServiceTable.selectByBusinessKey(businessKey,connection);

                // commit the transaction
                txn.commit();
            } catch (Exception ex) {
                try {
                    txn.rollback();
                } catch (java.sql.SQLException sqlex) {
                    sqlex.printStackTrace();
                }
                throw ex;
            }
        }
    }
}
