/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.fthpis.datatype.session.SessionEntity;
import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class SessionEntityTable {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(SessionEntityTable.class);

    static String insertSQL = null;
    static String deleteSQL = null;
    static String selectSQL = null;
    static String selectByPublisherSQL = null;
    static String verifyOwnershipSQL = null;
    static String selectPublisherSQL = null;

    static {
        // buffer used to build SQL statements
        StringBuffer sql = null;

        // build insertSQL
        sql = new StringBuffer(150);
        sql.append("INSERT INTO SESSION_ENTITY (");
        sql.append("SESSION_KEY,");
        sql.append("AUTHORIZED_NAME,");
        sql.append("PUBLISHER_ID,");
        sql.append("OPERATOR,");
        sql.append("LAST_UPDATE) ");
        sql.append("VALUES (?,?,?,?,?)");
        insertSQL = sql.toString();

        // build deleteSQL
        sql = new StringBuffer(100);
        sql.append("DELETE FROM SESSION_ENTITY ");
        sql.append("WHERE SESSION_KEY=?");
        deleteSQL = sql.toString();

        // build selectSQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("AUTHORIZED_NAME,");
        sql.append("OPERATOR ");
        sql.append("FROM SESSION_ENTITY ");
        sql.append("WHERE SESSION_KEY=?");
        selectSQL = sql.toString();

        // build selectByPublisherIDSQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("SESSION_KEY ");
        sql.append("FROM SESSION_ENTITY ");
        sql.append("WHERE PUBLISHER_ID=?");
        selectByPublisherSQL = sql.toString();

        // build verifyOwnershipSQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("* ");
        sql.append("FROM SESSION_ENTITY ");
        sql.append("WHERE SESSION_KEY=? ");
        sql.append("AND PUBLISHER_ID=?");
        verifyOwnershipSQL = sql.toString();

        // build selectPublisherSQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("PUBLISHER_ID ");
        sql.append("FROM SESSION_ENTITY ");
        sql.append("WHERE SESSION_KEY=?");
        selectPublisherSQL = sql.toString();
    }

    /**
     * Insert new row into the SESSION_ENTITY table.
     *
     * @param session object holding values to be inserted
     * @param publisherID
     * @param connection connection
     * @throws java.sql.SQLException
     */
    public static void insert(SessionEntity session,
                              String publisherID,
                              Connection connection) throws java.sql.
            SQLException {
        PreparedStatement statement = null;
        Timestamp timeStamp = new Timestamp(System.currentTimeMillis());

        try {
            statement = connection.prepareStatement(insertSQL);
            statement.setString(1, session.getSessionKey());
            statement.setString(2, session.getAuthorizedName());
            statement.setString(3, publisherID);
            statement.setString(4, session.getOperator());
            statement.setTimestamp(5, timeStamp);

            log.info("insert into SESSION_ENTITY table:\n\n\t" + insertSQL +
                     "\n\t SESSION_KEY=" + session.getSessionKey() +
                     "\n\t AUTHORIZED_NAME=" + session.getAuthorizedName() +
                     "\n\t PUBLISHER_ID=" + publisherID +
                     "\n\t OPERATOR=" + session.getOperator() +
                     "\n\t LAST_UPDATE=" + timeStamp.getTime() + "\n");

            // execute
            statement.executeUpdate();
        } finally {
            try {
                statement.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Insert SessionEntity PreparedStatement: " +
                        e.getMessage());
            }
        }
    }

    /**
     * Delete row from the SESSION_ENTITY table.
     *
     * @param sessionKey key value
     * @param  connection JDBC Connection
     * @throws java.sql.SQLException
     */
    public static void delete(String sessionKey, Connection connection) throws
            java.sql.SQLException {
        PreparedStatement statement = null;

        try {
            // prepare the delete
            statement = connection.prepareStatement(deleteSQL);
            statement.setString(1, sessionKey.toString());

            log.info("delete from SESSION_ENTITY table:\n\n\t" + deleteSQL +
                     "\n\t SESSION_KEY=" + sessionKey.toString() + "\n");

            // execute
            statement.executeUpdate();
        } finally {
            try {
                statement.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Delete SessionEntity PreparedStatement: " +
                        e.getMessage());
            }
        }
    }

    /**
     * Select one row from the SESSION_ENTITY table.
     *
     * @param sessionKey key value
     * @param  connection JDBC Connection
     * @throws java.sql.SQLException
     */
    public static SessionEntity select(String sessionKey,
                                       Connection connection) throws
            java.sql.SQLException {
        SessionEntity session = null;
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            statement = connection.prepareStatement(selectSQL);
            statement.setString(1, sessionKey.toString());

            log.info("select from SESSION_ENTITY table:\n\n\t" + selectSQL +
                     "\n\t SESSION_KEY=" + sessionKey.toString() + "\n");

            resultSet = statement.executeQuery();
            if (resultSet.next()) {
                session = new SessionEntity();
                session.setSessionKey(sessionKey);
                session.setAuthorizedName(resultSet.getString(1)); //("AUTHORIZED_NAME"));
                session.setOperator(resultSet.getString(2)); //("OPERATOR"));
            }

            return session;
        } finally {
            try {
                resultSet.close();
                statement.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Select SessionEntity ResultSet and PreparedStatement: " +
                        e.getMessage());
            }
        }
    }

    /**
     * Select all BusinessKeys from the business_entities table for a given
     * 'publisherID' value.
     *
     * @param publisherID The user id of the SessionEntity owner.
     * @param connection JDBC connection
     * @return Vector A Vector of BusinessKeys
     * @throws java.sql.SQLException
     */
    public static Vector selectByPublisherID(String publisherID,
                                             Connection connection) throws java.
            sql.SQLException {
        Vector keyList = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            // create a statement to query with
            statement = connection.prepareStatement(selectByPublisherSQL);
            statement.setString(1, publisherID);

            log.info("select from SESSION_ENTITY table:\n\n\t" +
                     selectByPublisherSQL +
                     "\n\t PUBLISHER_ID=" + publisherID + "\n");

            // execute the statement
            resultSet = statement.executeQuery();
            while (resultSet.next()) {
                keyList.add(resultSet.getString(1)); //("SESSION_KEY"));
            }

            return keyList;
        } finally {
            try {
                resultSet.close();
                statement.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Select SessionEntity ResultSet and PreparedStatement: " +
                        e.getMessage());
            }
        }
    }

    /**
     * Verify that 'publisherID' has the authority to update or delete
     * SessionEntity identified by the businessKey parameter
     *
     * @param  sessionKey
     * @param  publisherID
     * @param  connection
     * @throws java.sql.SQLException
     */
    public static boolean verifyOwnership(String sessionKey,
                                          String publisherID,
                                          Connection connection) throws java.
            sql.SQLException {
        if ((sessionKey == null) || (publisherID == null)) {
            return false;
        }

        boolean authorized = false;
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            statement = connection.prepareStatement(verifyOwnershipSQL);
            statement.setString(1, sessionKey);
            statement.setString(2, publisherID);

            log.info("checking ownership of SESSION_ENTITY:\n\n\t" +
                     verifyOwnershipSQL +
                     "\n\t SESSION_KEY=" + sessionKey +
                     "\n\t PUBLISHER_ID=" + publisherID + "\n");

            resultSet = statement.executeQuery();
            if (resultSet.next()) {
                authorized = true;
            }

            return authorized;
        } finally {
            try {
                resultSet.close();
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     * Return the 'publisherID' for the SessionEntity identified by the
     * businessKey parameter. Retusn null if the business entity key does
     * not represent a valid BuisnessEntity or if no publisherID is specified
     * for that particular SessionEntity.
     *
     * @param sessionKey
     * @param connection
     * @return publisherID or null if no publisherID is available.
     * @throws java.sql.SQLException
     */
    public static String selectPublisherID(String sessionKey,
                                           Connection connection) throws java.
            sql.SQLException {
        if (sessionKey == null) {
            return null;
        }

        String publisherID = null;
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            statement = connection.prepareStatement(selectPublisherSQL);
            statement.setString(1, sessionKey);

            log.info("fetching publishers ID for SESSION_ENTITY:\n\n\t" +
                     selectPublisherSQL +
                     "\n\t SESSION_KEY=" + sessionKey + "\n");

            resultSet = statement.executeQuery();
            if (resultSet.next()) {
                publisherID = resultSet.getString(1); //("PUBLISHER_ID");
            }

            return publisherID;
        } finally {
            try {
                resultSet.close();
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection("wscontext");
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {
        Transaction txn = new Transaction();
        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

        try {
            String sessionKey = uuidgen.uuidgen_with_timestamp();
            SessionEntity session = new SessionEntity();
            session.setSessionKey(sessionKey);
            session.setAuthorizedName("Mehmet Aktas");
            session.setOperator("www.jUDDI.org");

            String publisherID = "uddi user";

            // begin a new transaction
            txn.begin(connection);

            // insert a new SessionEntity
            SessionEntityTable.insert(session, publisherID, connection);

            // select one of the SessionEntity objects
            session = SessionEntityTable.select(sessionKey, connection);

            // delete that SessionEntity object
            //SessionEntityTable.delete(businessKey,connection);

            // re-select that SessionEntity object
            session = SessionEntityTable.select(sessionKey, connection);

            // commit the transaction
            txn.commit();
        } catch (Exception ex) {
            try {
                txn.rollback();
            } catch (java.sql.SQLException sqlex) {
                sqlex.printStackTrace();
            }
            throw ex;
        }
    }
}
