/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.fthpis.datatype.session.SessionEntity;
import cgl.fthpis.datatype.service.BusinessService;
import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.SessionKey;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class SessionServiceTable
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(SessionServiceTable.class);

  static String insertSQL = null;
  static String deleteSQL = null;
  static String selectSQL = null;
  static String selectBySessionKeySQL = null;
  static String deleteBySessionKeySQL = null;
  static String verifyOwnershipSQL = null;

  static
  {
    // buffer used to build SQL statements
    StringBuffer sql = null;

    // build insertSQL
    sql = new StringBuffer(100);
    sql.append("INSERT INTO SESSION_SERVICE (");
    sql.append("SESSION_KEY,");
    sql.append("SERVICE_KEY,");
    sql.append("LAST_UPDATE) ");
    sql.append("VALUES (?,?,?)");
    insertSQL = sql.toString();

    // build deleteSQL
    sql = new StringBuffer(100);
    sql.append("DELETE FROM SESSION_SERVICE ");
    sql.append("WHERE SERVICE_KEY=?");
    deleteSQL = sql.toString();

    // build selectSQL
    sql = new StringBuffer(100);
    sql.append("SELECT ");
    sql.append("SESSION_KEY ");
    sql.append("FROM SESSION_SERVICE ");
    sql.append("WHERE SERVICE_KEY=?");
    selectSQL = sql.toString();

    // build selectByBusinessKeySQL
    sql = new StringBuffer(200);
    sql.append("SELECT ");
    sql.append("SERVICE_KEY ");
    sql.append("FROM SESSION_SERVICE ");
    sql.append("WHERE SESSION_KEY=?");
    selectBySessionKeySQL = sql.toString();

    // build deleteByBusinessKeySQL
    sql = new StringBuffer(100);
    sql.append("DELETE FROM SESSION_SERVICE ");
    sql.append("WHERE SESSION_KEY=?");
    deleteBySessionKeySQL = sql.toString();

    // build verifyOwnershipSQL
    sql = new StringBuffer(200);
    sql.append("SELECT ");
    sql.append("* ");
    sql.append("FROM SESSION_ENTITY e, SESSION_SERVICE s ");
    sql.append("WHERE e.SESSION_KEY = s.SESSION_KEY ");
    sql.append("AND s.SERVICE_KEY=? ");
    sql.append("AND e.PUBLISHER_ID=?");
    verifyOwnershipSQL = sql.toString();
  }

  /**
   * Insert new row into the SESSION_SERVICE table.
   *
   * @param service object holding values to be inserted
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static void insert(BusinessService service,
                            Connection connection)
    throws java.sql.SQLException
  {
    log.info("cgl.fthpis.datastore.jdbc.SessionService.insert() function called.");
    PreparedStatement statement = null;
    Timestamp timeStamp = new Timestamp(System.currentTimeMillis());
    Vector sessionKeyVector = service.getSessionKeyVector();
    if (sessionKeyVector != null && sessionKeyVector.size() != 0) {
        for (Iterator iter = sessionKeyVector.iterator(); iter.hasNext(); ) {
            String sessionKey = (String) iter.next();
            if (sessionKey != null && service.getServiceKey() != null) {
                try {
                    statement = connection.prepareStatement(insertSQL);
                    statement.setString(1, sessionKey);
                    statement.setString(2, service.getServiceKey().toString());
                    statement.setTimestamp(3, timeStamp);

                    log.debug("insert into SESSION_SERVICE table:\n\n\t" +
                              insertSQL +
                              "\n\t SESSION_KEY=" +
                              sessionKey +
                              "\n\t SERVICE_KEY=" +
                              service.getServiceKey().toString() +
                              "\n\t LAST_UPDATE=" + timeStamp.getTime() + "\n");

                    statement.executeUpdate();
                } finally {
                    try {
                        statement.close();
                    } catch (Exception e) { /* ignored */}
                }
            }
        }
    }
  }

  /**
   * Delete row from the SESSION_SERVICE table.
   *
   * @param serviceKey Primary key value
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static void delete(String serviceKey,
                            Connection connection)
    throws java.sql.SQLException
  {
    PreparedStatement statement = null;

    try
    {
      // prepare
      statement = connection.prepareStatement(deleteSQL);
      statement.setString(1,serviceKey.toString());

      log.debug("delete from SESSION_SERVICE table:\n\n\t" + deleteSQL +
        "\n\t SERVICE_KEY=" + serviceKey.toString() + "\n");

      // execute
      statement.executeUpdate();
    }
    finally
    {
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Select one row from the SESSION_SERVICE table.
   *
   * @param serviceKey primary key value
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static BusinessService select(String serviceKey,
                                       Connection connection)
    throws java.sql.SQLException
  {
    BusinessService service = null;
    PreparedStatement statement = null;
    ResultSet resultSet = null;

    try
    {
      statement = connection.prepareStatement(selectSQL);
      statement.setString(1,serviceKey.toString());

      log.info("select from SESSION_SERVICE table:\n\n\t" + selectSQL +
        "\n\t SERVICE_KEY=" + serviceKey.toString() + "\n");

      resultSet = statement.executeQuery();
      service = new BusinessService();
      Vector vec = new Vector();
      if (resultSet.next())
      {
        vec.add(resultSet.getString(1));//("SESSION_KEY"));
      }
      service.setSessionKeyVector(vec);
      return service;
    }
    finally
    {
      try { resultSet.close(); } catch (Exception e) { /* ignored */ }
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Delete multiple rows from the SESSION_SERVICE table that are assigned to
   * the sessionKey specified.
   *
   * @param sessionKey String
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static void deleteBySessionKey(String sessionKey,
                                        Connection connection)
    throws java.sql.SQLException
  {
    PreparedStatement statement = null;

    try
    {
      // prepare the delete
      statement = connection.prepareStatement(deleteBySessionKeySQL);
      statement.setString(1,sessionKey.toString());

      log.debug("delet from the SESSION_SERVICE table:\n\n\t" + deleteBySessionKeySQL +
        "\n\t SESSION_KEY=" + sessionKey.toString() + "\n");

      // execute
      statement.executeUpdate();
    }
    finally
    {
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Select all rows from the business_service table for
   * a given sessionKey.
   *
   * @param sessionKey String
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static Vector selectBySessionKey(String sessionKey,
                                          Connection connection)
    throws java.sql.SQLException
  {
    Vector serviceList = new Vector();
    PreparedStatement statement = null;
    ResultSet resultSet = null;

    try
    {
     // create a statement to query with
      statement = connection.prepareStatement(selectBySessionKeySQL);
      statement.setString(1,sessionKey.toString());

      log.debug("select from SESSION_SERVICE table:\n\n\t" + selectBySessionKeySQL +
        "\n\t SESSION_KEY=" + sessionKey.toString() + "\n");

      // execute the statement
      resultSet = statement.executeQuery();
      while (resultSet.next())
      {
        BusinessService service = new BusinessService();
        SessionKey session_key = new SessionKey();
        session_key.setValue(sessionKey);
        Vector vec = new Vector();
        vec.add(session_key);
        service.setSessionKeyVector(vec);
        service.setServiceKey(resultSet.getString(1));//("SERVICE_KEY"));

        serviceList.add(service);
      }

      return serviceList;
    }
    finally
    {
      try { resultSet.close(); } catch (Exception e) { /* ignored */ }
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Verify that 'authorizedName' has the authority to update or delete
   * BusinessService identified by the serviceKey parameter
   *
   * @param serviceKey
   * @param publisherID
   * @param connection
   * @throws java.sql.SQLException
   */
  public static boolean verifyOwnership(String serviceKey,
                                        String publisherID,
                                        Connection connection)
    throws java.sql.SQLException
  {
    if ((serviceKey == null) || (publisherID == null))
      return false;

    boolean authorized = false;
    PreparedStatement statement = null;
    ResultSet resultSet = null;

    try
    {
      statement = connection.prepareStatement(verifyOwnershipSQL);
      statement.setString(1,serviceKey);
      statement.setString(2,publisherID);

      log.debug("checking ownership of SESSION_SERVICE:\n\n\t" + verifyOwnershipSQL +
        "\n\t SERVICE_KEY=" + serviceKey +
        "\n\t PUBLISHER_ID=" + publisherID + "\n");

      resultSet = statement.executeQuery();
      if (resultSet.next())
        authorized = true;

      return authorized;
    }
    finally
    {
      try { resultSet.close(); } catch (Exception e) { /* ignored */ }
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
    throws Exception
  {
    // make sure we're using a DBCP DataSource and
    // not trying to use JNDI to aquire one.
    Config.setStringProperty("juddi.useConnectionPool","true");

    Connection conn = null;
    try {
      conn = ConnectionManager.aquireConnection("wscontext");
      test(conn);
    }
    finally {
      if (conn != null)
        conn.close();
    }
  }

  public static void test(Connection connection)
    throws Exception
  {
    Transaction txn = new Transaction();
    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

    if (connection != null)
    {
      try
      {
        String sessionKey = uuidgen.uuidgen_with_timestamp();

        SessionEntity session = new SessionEntity();
        session.setSessionKey(sessionKey);
        session.setAuthorizedName("maktas");
        session.setOperator("CGL");

        String serviceKey = uuidgen.uuidgen();
        BusinessService service = new BusinessService();
        Vector sessionKeyVector = new Vector();
        sessionKeyVector.add(sessionKey);
        service.setSessionKeyVector(sessionKeyVector);
        service.setServiceKey(serviceKey);

        // begin a new transaction
        txn.begin(connection);

        String authorizedUserID = "maktas";

        // insert a new BusinessEntity
        SessionEntityTable.insert(session,authorizedUserID,connection);

        // insert a new SessionService
        SessionServiceTable.insert(service,connection);

        // insert another new SessionService
        service.setServiceKey(uuidgen.uuidgen());
        SessionServiceTable.insert(service,connection);

        // insert one more new SessionService
        service.setServiceKey(uuidgen.uuidgen());
        SessionServiceTable.insert(service,connection);

        // select a SessionService object
        service = SessionServiceTable.select(serviceKey,connection);

//        // delete a SessionService object
//        SessionServiceTable.delete(serviceKey,connection);
//
//        // select a SessionService object
//        service = SessionServiceTable.select(serviceKey,connection);
//
//        // select a Collection SessionService objects by SessionKey
//        SessionServiceTable.selectBySessionKey(sessionKey,connection);
//
//        // delete a Collection SessionService objects by SessionKey
//        SessionServiceTable.selectBySessionKey(sessionKey,connection);
//
//        // select a Collection SessionService objects by SessionKey
//        SessionServiceTable.selectBySessionKey(sessionKey,connection);

        // commit the transaction
        txn.commit();
      }
      catch(Exception ex)
      {
        try { txn.rollback(); }
        catch(java.sql.SQLException sqlex) { sqlex.printStackTrace(); }
        throw ex;
      }
    }
  }
}
