/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datatype.session;

import java.util.Vector;

import org.apache.juddi.datatype.Description;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import cgl.fthpis.datatype.service.BusinessService;
import cgl.fthpis.datatype.service.BusinessServices;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.Attribute;

/**
 * "Top-level information manager for all of the information about a
 *  particular set of information related to a session unit"
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class SessionEntity implements RegistryObject
{
  private static Log log = LogFactory.getLog(SessionEntity.class);
  String sessionKey;
  String operator;
  String authorizedName;
  Vector nameVector;
  Vector descVector;
  BusinessServices sessionServices;
  AttributeBag contextBag;
  Lease lease;

  /**
   * Constructs a new sessionEntity instance.
   */
  public SessionEntity()
  {
      log.info("cgl.fthpis.datatype.session.SessionEntity() constructor called.");
  }

  /**
   * Sets the session key of this session entity to the given key.
   * @param key The new key of this session entity.
   */
  public void setSessionKey(String key)
  {
    this.sessionKey = key;
  }

  /**
   * Returns the session key of this session entity.
   * @return The session key of this session entity.
   */
  public String getSessionKey()
  {
    return this.sessionKey;
  }

  /**
   * Sets the authorized name of this session entity to the given name.
   * @param name The new authorized name of this session entity.
   */
  public void setAuthorizedName(String name)
  {
    this.authorizedName = name;
  }

  /**
   * Returns the authorized name of this session entity.
   * @return The authorized name of this session entity, or null if
   *  this session entity doesn't have an authorized name.
   */
  public String getAuthorizedName()
  {
    return this.authorizedName;
  }

  /**
   * Sets the operator of this session entity to the given operator.
   * @param operator The new operator of this session entity.
   */
  public void setOperator(String operator)
  {
    this.operator = operator;
  }

  /**
   * Returns the operator of this session entity.
   * @return The operator of this session entity, or null if this session entity
   * doesn't have an operator.
   */
  public String getOperator()
  {
    return this.operator;
  }

  /**
   * Adds the given description.
   *
   * @param descr The description to add.
   */
  public void addDescription(Description descr)
  {
    // just return if the Description parameter is null (nothing to add)
    if (descr == null)
      return;

    if (this.descVector == null)
      this.descVector = new Vector();
    this.descVector.add(descr);
  }

  /**
   * Sets the description list to the current one.
   *
   * @param descr Collection of Description objects to set
   */
  public void setDescriptionVector(Vector descr)
  {
    this.descVector = descr;
  }

  /**
   * Returns the descriptions.
   *
   * @return the descriptions. If the aren't any descriptions, an empty
   *  enumeration is returned.
   */
  public Vector getDescriptionVector()
  {
    return this.descVector;
  }

  /**
   * Add a name to this session entity.
   *
   * @param name The Name to add.
   */
  public void addName(Name name)
  {
    // just return if name is null (nothing to add)
    if (name == null)
      return;

    if (this.nameVector == null)
      this.nameVector = new Vector();
    this.nameVector.add(name);
  }

  /**
   * Set the set of names of this session entity to the given set.
   *
   * @param names The new set of contacts.
   */
  public void setNameVector(Vector names)
  {
    this.nameVector = names;
  }

  /**
   * Returns the names of this session entity.
   *
   * @return The names of thes session entity. If this session entity
   *  doesn't have any names, an empty set is returned.
   */
  public Vector getNameVector()
  {
    return this.nameVector;
  }

  /**
   * Add a session service to this session entity.
   *
   * @param service The service to add.
   */
  public void addSessionService(BusinessService service)
  {
    // just return if service is null (nothing to add)
    if (service == null)
      return;

    // make sure the SessionServices has been initialized
    if (this.sessionServices == null)
      this.sessionServices = new BusinessServices();
    this.addSessionService(service);
  }

  /**
   * Sets session services to this session services list.
   * @param services The set of session services to add.
   */
  public void setSessionServices(BusinessServices services)
  {
    this.sessionServices = services;
  }

  /**
   * Returns the session services of this session entity.
   *
   * @return The session services of this session entity. If this
   *  session entity doesn't have any services, an empty enumeration is
   *  returned.
   */
  public BusinessServices getSessionServices()
  {
    return this.sessionServices;
  }

  /**
   * Add a context to this session.
   *
   * @param binding The context to add.
   */
  public void addContext(Attribute context)
  {
    if (this.contextBag == null)
      this.contextBag = new AttributeBag();
    this.contextBag.addAttribute(context);
  }

  /**
   * Returns the contextBag of this session.
   *
   * @return ThecontextBag of this session.
   */
  public AttributeBag getContextBag()
  {
    return this.contextBag;
  }

  /**
   * Sets the contextBag of this session.
   */
  public void setContextBag(AttributeBag contextBag)
  {
    this.contextBag = contextBag;
  }

  /**
   * Returns the lease of this session.
   *
   * @return The lease of this session.
   */
  public Lease getLease()
  {
    return this.lease;
  }

  /**
   * Sets the lease of this session.
   */
  public void setLease(Lease lease)
  {
    this.lease = lease;
  }

}
