/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.function;

import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.request.FindQualifier;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.UnsupportedException;
import org.apache.juddi.util.Config;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.error.TooManyOptionsException;
import org.apache.juddi.error.NameTooLongException;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.WSContextDataStoreFactory;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.request.FindContext;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.response.AttributeList;
import cgl.fthpis.datatype.response.AttributeInfos;
import cgl.fthpis.datatype.response.AttributeInfo;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class FindContextFunction extends AbstractFunction
{
  // private reference to FTHPIS logger
  private static Log log = LogFactory.getLog(FindContextFunction.class);

  /**
   *
   */
  public FindContextFunction(FTHPISRegistryEngine registry)
  {
    super(registry);
  }

  /**
   *
   */
  public RegistryObject execute(RegistryObject regObject)
    throws RegistryException
  {
    log.info("FindContextFunction.execute(RegistryObject regObject)");
    // extract individual parameters
    FindContext request = (FindContext)regObject;
    String generic = request.getGeneric();
    String serviceKey = request.getServiceKey();
    String sessionKey = request.getSessionKey();
    Vector nameVector = request.getNameVector();
    Lease lease = request.getLease();
    log.info("FindContextFunction.execute(RegistryObject regObject) => lease = " + lease);
    FindQualifiers qualifiers = request.getFindQualifiers();
    int maxRows = request.getMaxRows();
    Vector contextInfoVector = null;
    boolean truncatedResults = false;

             // first make sure we need to continue with this request. If
             // no arguments were passed in then we'll simply return
             // an empty ContextList (aka "a zero match result set").
             if(((serviceKey == null) || (serviceKey == "")) &&
                ((nameVector == null) || (nameVector.size() == 0))&&
                ((sessionKey == null) || (sessionKey == "")) &&
                (lease == null))
             {
               AttributeDetail detail = new AttributeDetail();
               detail.setGeneric(generic);
               detail.setContextVector(null);
               detail.setOperator(Config.getOperator());
               detail.setTruncated(false);
               return detail;
            }

            // aquire a FTHPIS datastore instance
            FTHPISDataStore dataStore = WSContextDataStoreFactory.getDataStore();

            try
            {
              dataStore.beginTrans();

            // validate the 'name' parameters as much as possible up-front before
            // calling into the data layer for relational validation.
            if (nameVector != null)
            {
              // only allowed to specify a maximum of 5 names (implementation
              // dependent).  This value is configurable.
              int maxNames = Config.getMaxNameElementsAllowed();
              if ((nameVector != null) && (nameVector.size() > maxNames))
                throw new TooManyOptionsException("max=" + maxNames);

              // names can not exceed the maximum character length specified by the
              // a max character length of 255 This value is configurable.
              int maxNameLength = Config.getMaxNameLengthAllowed();
              for (int i = 0; i < nameVector.size(); i++)
              {
                String name = ((Name) nameVector.elementAt(i)).getValue();
                if (name.length() > maxNameLength)
                  throw new NameTooLongException(name +
                      "(max=" + maxNameLength + ")");
              }
            }

//          // a find_context request MUST include a service_key attribute
//          if (((serviceKey == null) || (serviceKey.length() == 0)) &&
//              ((sessionKey == null) || (sessionKey.length() == 0)))
//              throw new InvalidKeyPassedException("serviceKey = " + serviceKey + " sessionKey = " + sessionKey);

          // validate the 'qualifiers' parameter as much as possible up-front before
          // calling into the data layer for relational validation.
          if (qualifiers != null) {
              Vector qVector = qualifiers.getFindQualifierVector();
              if ((qVector != null) && (qVector.size() > 0)) {
                  for (int i = 0; i < qVector.size(); i++) {
                      FindQualifier qualifier = (FindQualifier) qVector.elementAt(i);
                      String qValue = qualifier.getValue();

                      if ((!qValue.equals(FindQualifier.EXACT_NAME_MATCH)) &&
                          (!qValue.equals(FindQualifier.CASE_SENSITIVE_MATCH)) &&
                          (!qValue.equals(FindQualifier.OR_ALL_KEYS)) &&
                          (!qValue.equals(FindQualifier.OR_LIKE_KEYS)) &&
                          (!qValue.equals(FindQualifier.AND_ALL_KEYS)) &&
                          (!qValue.equals(FindQualifier.SORT_BY_NAME_ASC)) &&
                          (!qValue.equals(FindQualifier.SORT_BY_NAME_DESC)) &&
                          (!qValue.equals(FindQualifier.SORT_BY_DATE_ASC)) &&
                          (!qValue.equals(FindQualifier.SORT_BY_DATE_DESC)) &&
                          (!qValue.equals(FindQualifier.SERVICE_SUBSET)) &&
                          (!qValue.equals(FindQualifier.COMBINE_CATEGORY_BAGS))
                              )
                          throw new UnsupportedException("findQualifier=" + qValue);
                  }
              }
          }

          // perform the search for matching binding templates (return only keys in requested order)
          Vector keyVector = dataStore.findContext(sessionKey, serviceKey, nameVector, lease, qualifiers);
          log.info("FindContextFunction I");
          if ((keyVector != null) && (keyVector.size() > 0)) {
              // if a maxRows value has been specified and it's less than
              // the number of rows we are about to return then only return
              // maxRows specified.
              int rowCount = keyVector.size();
              if ((maxRows > 0) && (maxRows < rowCount)) {
                  rowCount = maxRows;
                  truncatedResults = true;
              }
              log.info("FindContextFunction II");
              // iterate through the binding templates keys fetching
              // each associated BindingTemplate in sequence.
              contextInfoVector = new Vector(rowCount);

              if (contextInfoVector != null || contextInfoVector.size() != 0) {
                  log.info("cgl.fthpis.function.FindContextFunction => contextVector.size() => " +
                           contextInfoVector.size());
                  log.info("cgl.fthpis.function.FindContextFunction => match properties keyVector size => " +
                           contextInfoVector.size());
              }
              log.info("FindContextFunction III");
              for (int i = 0; i < rowCount; i++) {
                  AttributeInfo attributeInfo = new AttributeInfo();
                  String attributeKey = (String)keyVector.elementAt(i);
                  attributeInfo.setAttributeKey(attributeKey);
                  log.info("FindContextFunction IV");
                  Attribute context = dataStore.fetchContext(attributeKey);
                  attributeInfo.setNameVector(context.getNameVector());

                  contextInfoVector.addElement(attributeInfo);
//                contextInfoVector.addElement(dataStore.fetchContextInfo((String)keyVector.elementAt(i)));
              }
          }

      dataStore.commit();
      log.info("FindContextFunction V");
      AttributeInfos contextInfos = new AttributeInfos();
      contextInfos.setAttributeInfoVector(contextInfoVector);
      log.info("FindContextFunction VI");
      AttributeList attributeList = new AttributeList();
      attributeList.setAttributeInfos(contextInfos);
      attributeList.setGeneric(generic);
      attributeList.setOperator(Config.getOperator());
      attributeList.setTruncated(truncatedResults);
      log.info("FindContextFunction VII");
      return attributeList;
    }
    catch(InvalidKeyPassedException keyex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(keyex.getMessage());
      keyex.printStackTrace();
      throw (RegistryException)keyex;
    }
    catch(UnsupportedException suppex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(suppex.getMessage());
      suppex.printStackTrace();
      throw (RegistryException)suppex;
    }
    catch(RegistryException regex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(regex.toString());
      regex.printStackTrace();
      throw (RegistryException)regex;
    }
    catch(Exception ex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(ex.toString());
      ex.printStackTrace();
      throw new RegistryException(ex);
    }
    finally
    {
      if (dataStore != null)
        dataStore.release();
    }
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
  {
    // initialize the registry
    FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
    reg.init();

    try
    {
            Lease context_lease = new Lease();
            //context_lease.setLeaseStartDate("2001-01-20T23:34:45Z");
            //context_lease.setLeaseExpirationDate("1111336085000");
            context_lease.setLeaseExpirationDate("0");

            Vector qualifierVector = new Vector();
            qualifierVector.add(new FindQualifier(FindQualifier.OR_ALL_KEYS));
            FindQualifiers qualifiers = new FindQualifiers();
            qualifiers.setFindQualifierVector(qualifierVector);

            // generate a Name Vector
            Vector nameVector = new Vector();
            Name name = new Name();
            name.setValue("%");
            nameVector.add(name);

            // create & execute the SaveSession request
            FindContext request = new FindContext();
            //request.setGeneric("session");
            //request.setSessionKey("656A4850-4508-11DA-BDFA-C3B7320D6066");
            //request.setServiceKey("A843EE10-4508-11DA-AB75-C1DB9A0EE37C");
            //request.setLease(context_lease);
            request.setFindQualifiers(qualifiers);
            request.setNameVector(nameVector);

            AttributeList response =
                      (AttributeList) (new FindContextFunction(reg).execute(request));

            AttributeInfos attributeInfos = response.getAttributeInfos();

            if (attributeInfos != null) {
                Vector attributeInfoVector = attributeInfos.getAttributeInfoVector();
                if (attributeInfoVector != null && attributeInfoVector.size() != 0) {
                    for (Iterator iter = attributeInfoVector.iterator(); iter.hasNext(); ) {
                        AttributeInfo attributeInfo = (AttributeInfo) iter.next();
                        log.info("context key = " + attributeInfo.getAttributeKey());
                    }

//                                Vector vector = response.getContextVector();
//                                int size = vector.size();
//                                if (vector != null) {
//                                    for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
//                                        Attribute context = (Attribute) iter.next();
//                                        log.info("context key = " + context.getAttributeKey());
//                                        log.info("context => session key = " + context.getSessionKey());
//                                        log.info("context => service key = " + context.getServiceKey());
//                                        log.info("context name = " + context.getAttributeName().getAttributeName());
//                                        log.info("context value = " + context.getAttributeValue().getAttributeValue());
//                                        log.info("context version = " + context.getAttributeVersion());
//                                        Lease lease = context.getLease();
//                                        if (lease != null) {
//                                            log.info("lease start date = " +
//                                                               lease.getLeaseStartDate());
//                                            log.info("lease end date = " +
//                                                               lease.getLeaseExpirationDate());
//                                  }
//                                    }
//                                 log.info("size = " + size);
//                                }
                }
            }
    }
    catch (Exception ex)
    {
      // write execption to the console
      ex.printStackTrace();
    }
    finally
    {
      // destroy the registry
      reg.dispose();
    }
  }
}
