/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.function;

import java.util.Vector;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.ExtendedUDDIDataStoreFactory;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.request.FindServiceAttribute;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.response.AttributeInfos;
import cgl.fthpis.datatype.response.AttributeList;
import cgl.fthpis.datatype.response.AttributeInfo;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.request.FindQualifier;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.UnsupportedException;
import org.apache.juddi.util.Config;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class FindServiceAttributeFunction extends AbstractFunction
{
  // private reference to FTHPIS logger
  private static Log log = LogFactory.getLog(FindServiceAttributeFunction.class);

  /**
   *
   */
  public FindServiceAttributeFunction(FTHPISRegistryEngine registry)
  {
    super(registry);
  }

  /**
   *
   */
  public RegistryObject execute(RegistryObject regObject)
    throws RegistryException
  {
    log.info("FindServiceAttributeFunction.execute(RegistryObject regObject)");
    // extract individual parameters
    FindServiceAttribute request = (FindServiceAttribute)regObject;
    String generic = request.getGeneric();
    String serviceKey = request.getServiceKey();
    CategoryBag categoryBag = request.getCategoryBag();
    Lease lease = request.getLease();
    log.info("FindServiceAttributeFunction.execute(RegistryObject regObject) => lease = " + lease);
    FindQualifiers qualifiers = request.getFindQualifiers();
    int maxRows = request.getMaxRows();

    // first make sure we need to continue with this request. If
    // no arguments were passed in then we'll simply return
    // an empty ServiceList (aka "a zero match result set").
    if(((categoryBag == null) || (categoryBag.size() == 0)) &&
       ((serviceKey == null) || (serviceKey == "")))
    {
      AttributeDetail detail = new AttributeDetail();
      detail.setGeneric(generic);
      detail.setAttributeVector(null);
      detail.setOperator(Config.getOperator());
      detail.setTruncated(false);
      return detail;
    }

    // aquire a FTHPIS datastore instance
    FTHPISDataStore dataStore = ExtendedUDDIDataStoreFactory.getDataStore();

    try
    {
      dataStore.beginTrans();

      // a find_binding request MUST include a service_key attribute
      if ((serviceKey == null) || (serviceKey.length() == 0))
        throw new InvalidKeyPassedException("serviceKey="+serviceKey);

      // validate the 'qualifiers' parameter as much as possible up-front before
      // calling into the data layer for relational validation.
      if (qualifiers != null)
      {
        Vector qVector = qualifiers.getFindQualifierVector();
        if ((qVector!=null) && (qVector.size() > 0))
        {
          for (int i=0; i<qVector.size(); i++)
          {
            FindQualifier qualifier = (FindQualifier)qVector.elementAt(i);
            String qValue = qualifier.getValue();

            if ((!qValue.equals(FindQualifier.EXACT_NAME_MATCH)) &&
                (!qValue.equals(FindQualifier.CASE_SENSITIVE_MATCH)) &&
                (!qValue.equals(FindQualifier.OR_ALL_KEYS)) &&
                (!qValue.equals(FindQualifier.OR_LIKE_KEYS)) &&
                (!qValue.equals(FindQualifier.AND_ALL_KEYS)) &&
                (!qValue.equals(FindQualifier.SORT_BY_NAME_ASC)) &&
                (!qValue.equals(FindQualifier.SORT_BY_NAME_DESC)) &&
                (!qValue.equals(FindQualifier.SORT_BY_DATE_ASC)) &&
                (!qValue.equals(FindQualifier.SORT_BY_DATE_DESC)) &&
                (!qValue.equals(FindQualifier.SERVICE_SUBSET)) &&
                (!qValue.equals(FindQualifier.COMBINE_CATEGORY_BAGS))
                )
              throw new UnsupportedException("findQualifier="+qValue);
          }
        }
      }

      Vector attributeInfoVector = null;
      boolean truncatedResults = false;

      // perform the search for matching binding templates (return only keys in requested order)
      Vector keyVector = dataStore.findServiceAttribute(serviceKey,categoryBag,lease,qualifiers);
      if ((keyVector != null) && (keyVector.size() > 0))
      {
        // if a maxRows value has been specified and it's less than
        // the number of rows we are about to return then only return
        // maxRows specified.
        int rowCount = keyVector.size();
        if ((maxRows > 0) && (maxRows < rowCount))
        {
          rowCount = maxRows;
          truncatedResults = true;
        }

        // iterate through the binding templates keys fetching
        // each associated BindingTemplate in sequence.
        attributeInfoVector = new Vector(rowCount);

        if (attributeInfoVector != null || attributeInfoVector.size() != 0) {
          log.info("org.apache.juddi.function.FindServiceAttributeFunction => attributeVector.size() => " + attributeInfoVector.size());
          log.info("org.apache.juddi.function.FindServiceAttributeFunction => match properties keyVector size => " + keyVector.size());
        }
        for (int i=0; i<rowCount; i++) {
            AttributeInfo attributeInfo = new AttributeInfo();
            attributeInfo.setAttributeKey((String)keyVector.elementAt(i));
            attributeInfoVector.add(attributeInfo);
//          attributeInfoVector.addElement(dataStore.fetchAttributeInfo((String)keyVector.elementAt(i)));
        }
      }


      dataStore.commit();

      // create a new SessionInfos instance and stuff
      // the new Vector of SessionInfos into it.
      AttributeInfos attributeInfos = new AttributeInfos();
      attributeInfos.setAttributeInfoVector(attributeInfoVector);

      // create a new BindingDetail instance and stuff
      // the new bindingTemplateVector into it.
      // create a new BindingDetail instance and stuff
      // the new bindingTemplateVector into it.
      AttributeList attributeList = new AttributeList();
      attributeList.setAttributeInfos(attributeInfos);
      attributeList.setGeneric(generic);
      attributeList.setOperator(Config.getOperator());
      attributeList.setTruncated(truncatedResults);
      return attributeList;
    }
    catch(InvalidKeyPassedException keyex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(keyex.getMessage());
      keyex.printStackTrace();
      throw (RegistryException)keyex;
    }
    catch(UnsupportedException suppex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(suppex.getMessage());
      suppex.printStackTrace();
      throw (RegistryException)suppex;
    }
    catch(RegistryException regex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(regex.toString());
      regex.printStackTrace();
      throw (RegistryException)regex;
    }
    catch(Exception ex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(ex.toString());
      ex.printStackTrace();
      throw new RegistryException(ex);
    }
    finally
    {
      if (dataStore != null)
        dataStore.release();
    }
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
  {
    // initialize the registry
    FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
    reg.init();

    try
    {
    }
    catch (Exception ex)
    {
      // write execption to the console
      ex.printStackTrace();
    }
    finally
    {
      // destroy the registry
      reg.dispose();
    }
  }
}
