/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.function;

import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.request.FindQualifier;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.error.NameTooLongException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.TooManyOptionsException;
import org.apache.juddi.error.UnsupportedException;
import org.apache.juddi.util.Config;
import org.apache.juddi.datatype.Description;
import org.apache.juddi.datatype.ServiceKey;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.WSContextDataStoreFactory;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.request.FindSession;
import cgl.fthpis.datatype.response.SessionInfos;
import cgl.fthpis.datatype.response.SessionInfo;
import cgl.fthpis.datatype.response.SessionList;
import cgl.fthpis.datatype.attribute.AttributeName;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeFindQualifier;
import cgl.fthpis.datatype.attribute.AttributeFindQualifiers;
import cgl.fthpis.datatype.attribute.AttributeValueType;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.request.GetSessionDetail;
import cgl.fthpis.datatype.response.SessionDetail;
import cgl.fthpis.datatype.session.SessionEntity;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.request.GetContextDetail;

/**
 * "This FindSession API call returns a sessionList on success. This
 * structure contains information about each matching session In the event that
 * no matches were located for the specified criteria, a sessionList
 * structure with zero sessionInfo structures is returned. If no
 * arguments are passed, a zero-match result set will be returned."
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class FindSessionFunction extends AbstractFunction
{
  // private reference to FTHPIS logger
  private static Log log = LogFactory.getLog(FindSessionFunction.class);

  /**
   *
   */
  public FindSessionFunction(FTHPISRegistryEngine registry)
  {
    super(registry);
  }

  /**
   *
   */
  public RegistryObject execute(RegistryObject regObject)
    throws RegistryException
  {
    FindSession request = (FindSession) regObject;

    String generic = request.getGeneric();
    Vector serviceKeyVector = request.getServiceKeyVector();
    Vector nameVector = request.getNameVector();
    AttributeBag contextBag = request.getAttributeBag();
    Lease lease = request.getLease();

    FindQualifiers qualifiers = request.getFindQualifiers();
    int maxRows = request.getMaxRows();

    // make sure we need to continue with this request. If
    // no arguments were passed in then we'll simply return
    // an empty BusinessList (aka "a zero match result set").
    if (((serviceKeyVector == null) || (serviceKeyVector.size() == 0))
        && ((nameVector == null) || (nameVector.size() == 0))
        && ((contextBag == null) || (contextBag.size() == 0))
        && ((lease == null))
      )
    {
      SessionList list = new SessionList();
      list.setGeneric(generic);
      list.setSessionInfos(new SessionInfos());
      list.setOperator(Config.getOperator());
      list.setTruncated(false);
      return list;
    }

    // aquire a FTHPIS datastore instance
    FTHPISDataStore dataStore = WSContextDataStoreFactory.getDataStore();

    try
    {
      dataStore.beginTrans();

      // validate the 'name' parameters as much as possible up-front before
      // calling into the data layer for relational validation.
      if (nameVector != null)
      {
        // only allowed to specify a maximum of 5 names (implementation
        // dependent).  This value is configurable.
        int maxNames = Config.getMaxNameElementsAllowed();
        if ((nameVector != null) && (nameVector.size() > maxNames))
          throw new TooManyOptionsException("max=" + maxNames);

        // names can not exceed the maximum character length of 255).
        // This value is configurable.
        int maxNameLength = Config.getMaxNameLengthAllowed();
        for (int i = 0; i < nameVector.size(); i++)
        {
          String name = ((Name) nameVector.elementAt(i)).getValue();
          if (name.length() > maxNameLength)
            throw new NameTooLongException(name +
                "(max=" + maxNameLength + ")");
        }
      }

      // validate the 'qualifiers' parameter as much as possible up-front before
      // calling into the data layer for relational validation.
      if (qualifiers != null)
      {
        Vector qVector = qualifiers.getFindQualifierVector();
        if ((qVector!=null) && (qVector.size() > 0))
        {
          for (int i = 0; i < qVector.size(); i++)
          {
            FindQualifier qualifier = (FindQualifier) qVector.elementAt(i);
            String qValue = qualifier.getValue();

            if ((!qValue.equals(FindQualifier.EXACT_NAME_MATCH))
              && (!qValue.equals(FindQualifier.CASE_SENSITIVE_MATCH))
              && (!qValue.equals(FindQualifier.OR_ALL_KEYS))
              && (!qValue.equals(FindQualifier.OR_LIKE_KEYS))
              && (!qValue.equals(FindQualifier.AND_ALL_KEYS))
              && (!qValue.equals(FindQualifier.SORT_BY_NAME_ASC))
              && (!qValue.equals(FindQualifier.SORT_BY_NAME_DESC))
              && (!qValue.equals(FindQualifier.SORT_BY_DATE_ASC))
              && (!qValue.equals(FindQualifier.SORT_BY_DATE_DESC))
              && (!qValue.equals(FindQualifier.SERVICE_SUBSET))
              && (!qValue
                .equals(FindQualifier.COMBINE_CATEGORY_BAGS))
            )
              throw new UnsupportedException(
                "findQualifier=" + qValue);
          }
        }
      }

      Vector infoVector = null;
      boolean truncatedResults = false;

      // perform the search for matching session entities (returns only session keys in the order requested)
      Vector keyVector = dataStore.findSession(serviceKeyVector, nameVector, contextBag, lease, qualifiers);

      if ((keyVector != null) && (keyVector.size() > 0))
      {
        // if a maxRows value has been specified and it's less than
        // the number of rows we are about to return then only return
        // maxRows specified.
        int rowCount = keyVector.size();
        if ((maxRows > 0) && (maxRows < rowCount))
        {
          rowCount = maxRows;
          truncatedResults = true;
        }

        // iterate through the business entity keys fetching
        // each associated BusinessInfo in sequence.
        infoVector = new Vector(rowCount);
        for (int i = 0; i < rowCount; i++) {
            SessionInfo sessionInfo = new SessionInfo();

            String sessionKey = (String) keyVector.elementAt(i);
            sessionInfo.setSessionKey(sessionKey);

            log.info("FindSessionFunction 1");
            //additional cost
            SessionEntity session = dataStore.fetchSession(sessionKey);
            sessionInfo.setNameVector(session.getNameVector());

            log.info("FindSessionFunction 2");

            infoVector.addElement(sessionInfo);
//            infoVector.addElement(dataStore.fetchSessionInfo((String) keyVector.elementAt(i)));
        }
      }

      dataStore.commit();

      // create a new SessionInfos instance and stuff
      // the new Vector of SessionInfos into it.
      SessionInfos infos = new SessionInfos();
      infos.setSessionInfoVector(infoVector);

      // create a new SessionList instance and stuff
      // the new SessionInfos instance into it.
      SessionList list = new SessionList();
      list.setSessionInfos(infos);
      list.setGeneric(generic);
      list.setOperator(Config.getOperator());
      list.setTruncated(truncatedResults);
      return list;
    }
    catch(TooManyOptionsException tmoex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.info(tmoex.getMessage());
      throw (RegistryException)tmoex;
    }
    catch(NameTooLongException ntlex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.info(ntlex.getMessage());
      throw (RegistryException)ntlex;
    }
    catch(UnsupportedException suppex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.info(suppex.getMessage());
      throw (RegistryException)suppex;
    }
    catch(RegistryException regex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.error(regex);
      throw (RegistryException)regex;
    }
    catch(Exception ex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.error(ex);
      throw new RegistryException(ex);
    }
    finally
    {
      if (dataStore != null)
        dataStore.release();
    }
  }

  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/

  public static void main(String[] args)
  {
    // initialize the registry
    FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
    reg.init();

    try
    {
      // create a request


      Vector qualifierVector = new Vector();
      qualifierVector.add(new FindQualifier(FindQualifier.AND_ALL_KEYS));
      FindQualifiers qualifiers = new FindQualifiers();
      qualifiers.setFindQualifierVector(qualifierVector);

      //FIRST CRITERIA
      Vector nameVector = new Vector();
      nameVector.addElement(new Name("GIS Session"));

      //SECOND CRITERIA
      Lease lease = new Lease();
      lease.setLeaseExpirationDate("1169336085000");

      //THIRD CRITERIA
      AttributeBag contextBag = new AttributeBag();
      Attribute context =  new Attribute();
      context.setAttributeName(new AttributeName("cpu_number"));
      context.setAttributeValueType(new AttributeValueType("number"));
      context.setAttributeValue(new AttributeValue("4"));
      context.setAttributeFindQualifier(new AttributeFindQualifier(AttributeFindQualifiers.EQUAL_TO));
      contextBag.addAttribute(context);

      //FOURTH CRITERIA
      Vector serviceKeyVector = new Vector();
      String key = "EB3BF060-4ADE-11DA-B4E8-BFA5D50BF4D7";
      ServiceKey serviceKey = new ServiceKey();
      serviceKey.setValue(key);
      serviceKeyVector.add(serviceKey);

      FindSession request = new FindSession();
      request.setServiceKeyVector(serviceKeyVector);
      request.setNameVector(nameVector);
      request.setAttributeBag(contextBag);
      request.setLease(lease);
      request.setMaxRows(10);
      request.setFindQualifiers(qualifiers);

      // invoke the server
      SessionList response = (SessionList) (new FindSessionFunction(reg).execute(request));

      SessionInfos infos = response.getSessionInfos();
      Vector vector = infos.getSessionInfoVector();
      if (vector != null) {
          for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
              SessionInfo item = (SessionInfo) iter.next();
              //log.info("session key = " + item.getSessionKey());
              //OK. I JUST SAVED IT LET'S GET THE DETAILS
              GetSessionDetail detail_request = new GetSessionDetail();
              Vector sessionKeyVector = new Vector();
              sessionKeyVector.add(item.getSessionKey());
              detail_request.setSessionKeyVector(sessionKeyVector);
              // invoke the server
              SessionDetail session_detail_response = (SessionDetail) (new GetSessionDetailFunction(reg).execute(detail_request));
              Vector sessionEntityVector = session_detail_response.getSessionEntityVector();
              for (Iterator iter0 = sessionEntityVector.iterator(); iter0.hasNext(); ) {
                  SessionEntity session = (SessionEntity) iter0.next();
                  log.info("session key = " + session.getSessionKey());
                  nameVector = session.getNameVector();
                  for (Iterator iter1 = nameVector.iterator(); iter1.hasNext(); ) {
                      Name name = (Name) iter1.next();
                      log.info("session name = " + name.getValue());
                  }
                  Vector descVector = session.getDescriptionVector();
                  for (Iterator iter2 = descVector.iterator(); iter2.hasNext(); ) {
                      Description desc = (Description) iter2.next();
                      log.info("session description = " + desc.getValue());
                    }
                    contextBag = session.getContextBag();
                    if (contextBag != null) {
                        Vector attributeVector = contextBag.getAttributeVector();
                        for (Iterator iter3 = attributeVector.iterator();
                                              iter3.hasNext(); ) {
                            context = (Attribute) iter3.next();
                            //log.info("context key = " + context.getAttributeKey());
                            GetContextDetail cntx_detail_request = new GetContextDetail();
                            Vector contextKeyVector = new Vector();
                            contextKeyVector.add(context.getAttributeKey());
                            cntx_detail_request.setContextKeyVector(contextKeyVector);
                            // invoke the server
                            AttributeDetail cntx_detail_response =
                                    (AttributeDetail) (new GetContextDetailFunction(reg).execute(cntx_detail_request));

                            Vector vec = cntx_detail_response.getContextVector();
                            for (Iterator iter1 = vec.iterator(); iter1.hasNext(); ) {
                                context = (Attribute) iter1.next();
                                log.info("context key = " + context.getAttributeKey());
                                log.info("context => session key = " + context.getSessionKey());
                                log.info("context => service key = " + context.getServiceKey());
                                log.info("context name = " + context.getAttributeName().getAttributeName());
                                log.info("context value = " + context.getAttributeValue().getAttributeValue());
                                log.info("context version = " + context.getAttributeVersion());
                                lease = context.getLease();
                                if (lease != null) {
                                    log.info("context lease start date = " +
                                                       lease.getLeaseStartDate());
                                    log.info("context lease end date = " +
                                                       lease.getLeaseExpirationDate());
                                  }
                            }
                         }
                    }
                    lease = session.getLease();
                    if (lease != null) {
                        log.info("session lease start date = " +
                                           lease.getLeaseStartDate());
                        log.info("session lease end date = " +
                                           lease.getLeaseExpirationDate());
                    }
      }


          }
      }
    }
    catch (Exception ex)
    {
      // write execption to the console
      ex.printStackTrace();
    }
    finally
    {
      // destroy the registry
      reg.dispose();
    }
  }
}
