/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.function;

import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.Description;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.UnsupportedException;
import org.apache.juddi.error.UserMismatchException;
import org.apache.juddi.util.Config;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.WSContextDataStoreFactory;
import cgl.fthpis.datatype.session.SessionEntity;
import cgl.fthpis.datatype.response.SessionDetail;
import cgl.fthpis.datatype.request.SaveSession;
import cgl.fthpis.datatype.publisher.Publisher;
import cgl.fthpis.datatype.request.AuthInfo;
import cgl.fthpis.datatype.response.AuthToken;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.attribute.AttributeName;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeValueType;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.function.GetAuthTokenForWSContextFunction;
import cgl.fthpis.datatype.request.GetSessionDetail;
import cgl.fthpis.datatype.request.GetContextDetail;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.request.GetAuthTokenForWSContext;
import cgl.fthpis.datatype.publisher.PublisherID;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class SaveSessionFunction extends AbstractFunction
{
  // private reference to FTHPIS logger
  private static Log log = LogFactory.getLog(SaveSessionFunction.class);

  /**
   *
   */
  public SaveSessionFunction(FTHPISRegistryEngine registry)
  {
    super(registry);
  }

  /**
   *
   */
  public RegistryObject execute(RegistryObject regObject)
    throws RegistryException
  {
    SaveSession request = (SaveSession)regObject;
    String generic = request.getGeneric();
    AuthInfo authInfo = request.getAuthInfo();
    Vector sessionVector = request.getSessionEntityVector();
    Vector uploadRegVector = request.getUploadRegisterVector();
    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

    // UploadRegistry functionality is not currently supported.
    if ((uploadRegVector != null) && (uploadRegVector.size() > 0))
      throw new UnsupportedException("Saving SessionEntities via " +
        "UploadRegistry is not supported.");

    //FTHPISDataStore uddiDataStore = DataStoreFactory.g

    // aquire a FTHPIS datastore instance
    FTHPISDataStore dataStore = WSContextDataStoreFactory.getDataStore();
    try
    {
      dataStore.beginTrans();

      // validate authentication parameters
      Publisher publisher = getPublisher(authInfo,dataStore);

      String publisherID = "";
      String authorizedName = "";

      if (publisher != null) {
         publisherID = publisher.getPublisherID();
         authorizedName = publisher.getName();
      }

      // validate request parameters & execute
      for (int i=0; i<sessionVector.size(); i++)
      {

        // move the SessionEntity into a form we can work with easily
        SessionEntity session = (SessionEntity)sessionVector.elementAt(i);

        String sessionKey = session.getSessionKey();

        // If a SessionKey was specified then make sure it's a valid one.
        if ((sessionKey != null) && (sessionKey.length() > 0) && (!dataStore.isValidSessionKey(sessionKey)))
          throw new InvalidKeyPassedException("sessionKey="+sessionKey);

        // If a SessionKey was specified then make sure 'publisherID' controls it.
        if ((sessionKey != null) && (sessionKey.length() > 0) && (!dataStore.isSessionPublisher(sessionKey,publisherID)))
          throw new UserMismatchException("sessionKey="+sessionKey);
      }

      for (int i=0; i<sessionVector.size(); i++)
      {
        // move the SessionEntity into a form we can work with easily
        SessionEntity session = (SessionEntity)sessionVector.elementAt(i);

        String sessionKey = session.getSessionKey();

        // If the new SessionEntity has a SessionKey then it must already
        // exists so delete the old one. It a SessionKey isn't specified then
        // this is a new SessionEntity so create a new SessionKey for it.
        if ((sessionKey != null) && (sessionKey.length() > 0))
        {
          dataStore.deleteSession(sessionKey);
        }
        else
        {
           session.setSessionKey("uuid:" + uuidgen.uuidgen_with_timestamp());
        }

        // Everything checks out so let's save it. First
        // store 'authorizedName' and 'operator' values
        // in each SessionEntity.
        session.setAuthorizedName(authorizedName);
        session.setOperator(Config.getOperator());
        // If no contacts were specified with the Session
        // Entity then add a new contact of type 'publisher'
        // using the publishers information.
        dataStore.saveSession(session,publisherID);
      }
      dataStore.commit();

      SessionDetail detail = new SessionDetail();
      detail.setGeneric(generic);
      detail.setOperator(Config.getOperator());
      detail.setTruncated(false);
      detail.setSessionEntityVector(sessionVector);
      return detail;
    }
    catch(UnsupportedException suppex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.info(suppex);
      throw (RegistryException)suppex;
    }
    catch(InvalidKeyPassedException ikpex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.info(ikpex);
      throw (RegistryException)ikpex;
    }
    catch(UserMismatchException umex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.info(umex);
      throw (RegistryException)umex;
    }
    catch(RegistryException regex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.error(regex);
      throw (RegistryException)regex;
    }
    catch(Exception ex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.error(ex);
      throw new RegistryException(ex);
    }
    finally
    {
      if (dataStore != null)
      	dataStore.release();

    }
  }

  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
  {
    // initialize the registry
    FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
    reg.init();


    try
    {
      GetAuthTokenForWSContext authTokenRequest = new GetAuthTokenForWSContext("maktas","changeIt");
      AuthToken authToken = (AuthToken)(new GetAuthTokenForWSContextFunction(reg).execute(authTokenRequest));
      log.info("Function: getAuthToken(userid/password)");
      log.info("AuthInfo: "+ authToken.getAuthInfo());
      AuthInfo authInfo = authToken.getAuthInfo();

      // generate a SessionEntity
      SessionEntity sessionEntity = new SessionEntity();

      sessionEntity.setSessionKey(null);

      // generate a Name Vector
      Vector nameVector = new Vector();
      nameVector.add(new Name("GIS Session"));
      sessionEntity.setNameVector(nameVector);

      // generate a Description Vector
      Vector descVector = new Vector();
      descVector.add(new Description("This is a GIS Session"));
      sessionEntity.setDescriptionVector(descVector);

      // generate a AttributeBag
      AttributeBag attributeBag = new AttributeBag();
      Attribute context = new Attribute();
      PublisherID id = new PublisherID();
      id.setValue(authInfo.getPublisherID());
      context.setAttributeOwnerPublisherID(id);
      context.setAttributeName(new AttributeName("cpu_number"));
      context.setAttributeValueType(new AttributeValueType("number"));
      context.setAttributeValue(new AttributeValue("6"));
      Lease context_lease = new Lease();
      //context_lease.setLeaseStartDate("2001-01-20T23:34:45Z");
      context_lease.setIsInfinite("false");
      context_lease.setLeaseTimeOut("1000000000000");
      context.setLease(context_lease);
      attributeBag.addAttribute(context);
      sessionEntity.setContextBag(attributeBag);

      // generate a Lease
      Lease lease = new Lease();
      lease.setIsInfinite("false");
      lease.setLeaseTimeOut("1000000000000");
      sessionEntity.setLease(lease);
      // generate a SessionEntity Vector
      Vector sessionEntityVector = new Vector();
      sessionEntityVector.add(sessionEntity);
      // create & execute the SaveSession request
      SaveSession request = new SaveSession();

      request.setAuthInfo(authInfo);
      request.setSessionEntityVector(sessionEntityVector);

      SessionDetail detail = (SessionDetail)reg.execute(request);
      String session_key = "";
      if (detail != null) {
          Vector vector = detail.getSessionEntityVector();
          for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
              SessionEntity session = (SessionEntity) iter.next();
              //log.info("session key = " + session.getSessionKey());
              session_key = session.getSessionKey();
          }
      }

      //OK. I JUST SAVED IT LET'S GET THE DETAILS
      GetSessionDetail detail_request = new GetSessionDetail();
      Vector sessionKeyVector = new Vector();
      sessionKeyVector.add(session_key);
      detail_request.setSessionKeyVector(sessionKeyVector);
      // invoke the server
      SessionDetail response = (SessionDetail) (new GetSessionDetailFunction(reg).execute(detail_request));
      sessionEntityVector = response.getSessionEntityVector();
      for (Iterator iter = sessionEntityVector.iterator(); iter.hasNext(); ) {
          SessionEntity session = (SessionEntity) iter.next();
          log.debug("session key = " + session.getSessionKey());
          nameVector = session.getNameVector();
          for (Iterator iter1 = nameVector.iterator(); iter1.hasNext(); ) {
              Name name = (Name) iter1.next();
              log.info("name = " + name.getValue());
          }
          descVector = session.getDescriptionVector();
          for (Iterator iter2 = descVector.iterator(); iter2.hasNext(); ) {
              Description desc = (Description) iter2.next();
              log.info("description = " + desc.getValue());
            }
            AttributeBag contextBag = session.getContextBag();
            if (contextBag != null) {
                Vector attributeVector = contextBag.getAttributeVector();
                for (Iterator iter3 = attributeVector.iterator();
                                      iter3.hasNext(); ) {
                    context = (Attribute) iter3.next();
                    //log.info("context key = " + context.getAttributeKey());
                    GetContextDetail cntx_detail_request = new GetContextDetail();
                    Vector contextKeyVector = new Vector();
                    contextKeyVector.add(context.getAttributeKey());
                    cntx_detail_request.setContextKeyVector(contextKeyVector);
                    // invoke the server
                    AttributeDetail cntx_detail_response =
                            (AttributeDetail) (new GetContextDetailFunction(reg).execute(cntx_detail_request));

                    Vector vector = cntx_detail_response.getContextVector();
                    for (Iterator iter1 = vector.iterator(); iter1.hasNext(); ) {
                        context = (Attribute) iter1.next();
                        log.info("context key = " + context.getAttributeKey());
                        log.info("context => session key = " + context.getSessionKey());
                        log.info("context => service key = " + context.getServiceKey());
                        log.info("context name = " + context.getAttributeName());
                        log.info("context value = " + context.getAttributeValue().getAttributeValue());
                        log.info("context version = " + context.getAttributeVersion());
                        lease = context.getLease();
                        if (lease != null) {
                            log.info("context lease start date = " +
                                               lease.getLeaseStartDate());
                            log.info("context lease end date = " +
                                               lease.getLeaseExpirationDate());
                          }
                    }
                 }
            }
            lease = session.getLease();
            if (lease != null) {
                log.info("session lease start date = " +
                                   lease.getLeaseStartDate());
                log.info("session lease end date = " +
                                   lease.getLeaseExpirationDate());
            }
      }
    }
    catch (Exception ex)
    {
      // write execption to the console
      ex.printStackTrace();
    }
    finally
    {
      // destroy the registry
      reg.dispose();
    }
  }
}
